Package: SPARTAAS
Type: Package
Title: Statistical Pattern Recognition and daTing using Archaeological
        Artefacts assemblageS
Version: 1.2.5
Date: 2026-03-25
Authors@R: c(person("Arthur","Coulon", role = c("aut","cre"),
                     email = "arthur-coulon@outlook.fr"),
             person("Lise","Bellanger", role = "aut",
                     email = "lise.bellanger@univ-nantes.fr"),
             person("Philippe","Husi", role = "aut",
                     email = "philippe.husi@univ-tours.fr"),
             person("Bruno","Desachy", role = "ctb"),
             person("Benjamin","Martineau", role = "ctb")
             )
Maintainer: Arthur Coulon <arthur-coulon@outlook.fr>
URL: https://spartaas.gitpages.huma-num.fr/r-package/
Description: Statistical pattern recognition and dating using archaeological artefacts assemblages.
    Package of statistical tools for archaeology.
    hclustcompro()/perioclust(): Bellanger Lise, Coulon Arthur, Husi Philippe (2021, ISBN:978-3-030-60103-4).
    mapclust(): Bellanger Lise, Coulon Arthur, Husi Philippe (2021) <doi:10.1016/j.jas.2021.105431>.
    seriograph(): Desachy Bruno (2004) <doi:10.3406/pica.2004.2396>.
    cerardat(): Bellanger Lise, Husi Philippe (2012) <doi:10.1016/j.jas.2011.06.031>.
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.1.0)
Imports:
        FactoMineR,grDevices,dplyr,tidyr,ggplot2,plotly,stringr,colorspace,
        shiny,shinydashboard,shinyjs,shinyjqui,fpc,ggdendro,htmltools,htmlwidgets,
        shinythemes,explor,shinyWidgets,scatterD3,ks,grid,cluster,leaflet,ape,
        MASS,ade4,lmtest,nor1mix,shinycssloaders,scales,fastcluster,nloptr
Suggests: knitr, rmarkdown,sf
RoxygenNote: 7.1.1
NeedsCompilation: no
Packaged: 2026-03-26 09:01:43 UTC; arcoulon
Author: Arthur Coulon [aut, cre],
  Lise Bellanger [aut],
  Philippe Husi [aut],
  Bruno Desachy [ctb],
  Benjamin Martineau [ctb]
Repository: CRAN
Date/Publication: 2026-03-26 10:10:07 UTC
Built: R 4.6.0; ; 2026-04-25 14:09:45 UTC; unix
