export("CAdist","adjacency","arrondi","seriograph","serio_app","overlap","timerange",
       "cerardat","extract_results","cerardat_estim_nf","cerardat_app","arrangePlot",
       "hclustcompro","hclust","hclustcompro_cutree","hclustcompro_select_alpha",
       "perioclust","perioclust_subdivide","mapclust","mapclust_cut_tree",
       "mapclust_app","optim_select_alpha","hclustcompro_app")

importFrom("FactoMineR",      "CA")
importFrom("grDevices",       "colorRampPalette","recordPlot","rainbow","graphics.off","tiff","rgb")
importFrom("tidyr",           "gather")
importFrom("dplyr",           "mutate","group_by","summarise_all","funs","ungroup","inner_join","arrange",
                              "summarise","first","last","filter")
importFrom("ggplot2",         "ggplot","geom_bar","geom_col","aes","ggtitle","labs","xlab",
                              "ylab","facet_wrap","position_stack","coord_flip","facet_grid","scale_fill_manual",
                              "scale_y_continuous","theme","element_text","element_blank","scale_x_discrete",
                              "geom_segment","margin","geom_point","scale_colour_gradientn","geom_line",
                              "scale_x_continuous","geom_text","scale_y_reverse","ylim","element_line",
                              "geom_hline","geom_errorbar","theme_classic","geom_vline","scale_y_log10",
                              "annotation_logticks","annotate")
importFrom("ggdendro",        "dendro_data","ggdendrogram","segment","label")
importFrom("plotly",          "plot_ly","add_lines","add_segments","add_markers","layout","subplot")
importFrom("graphics",        "barplot","plot","abline","legend","lines","points","text","mtext",
                              "axis","boxplot","image","par","plot.new", "segments", "symbols","polygon")
importFrom("stats",           "median","quantile","cophenetic","cor","dist","as.dist","chisq.test","cutree",
                              "optimize","rect.hclust","reorder","coef","sd","lm","qnorm","na.omit",
                              "approx","density","predict","rstudent","shapiro.test","as.hclust","as.formula",
                              "formula","residuals","lm.influence")
importFrom("nor1mix",         "qnorMix","norMix","dnorMixL")
importFrom("lmtest",          "dwtest","bptest")
importFrom("MASS",            "boxcox")
importFrom("ade4",            "supcol","dudi.coa","inertia.dudi")
importFrom("utils",           "setTxtProgressBar","txtProgressBar","data","str","capture.output","head")
importFrom("stringr",         'str_sub','str_split','fixed','str_extract')
importFrom("colorspace",      "rainbow_hcl")
importFrom("shinydashboard",  "box")
importFrom("shinyjqui",       "jqui_resizable")
importFrom("fpc",             "cluster.stats")
importFrom("ks",              "kde","Hscv")
importFrom("cluster",         "silhouette")
importFrom("htmltools",       "htmlEscape")
importFrom("shinycssloaders", "withSpinner")
importFrom("fastcluster",     "hclust")
importFrom("scales",          "scientific")
importFrom("nloptr",          "nloptr")
importFrom("ape",             "Moran.I")

import("shinyjs",             except=c("runExample","alert"))

import('grid','shiny','leaflet','htmlwidgets','shinythemes',"shinyWidgets",
       "scatterD3","explor")

S3method("print",     "perioclust_cl")
S3method("print",     "selectAlpha_obj")
S3method("print",     "seriograph")
S3method("summary",   "selectAlpha_obj")
S3method("print",     "cerardat_obj")
S3method("plot",      "cerardat_obj")
