.onAttach <- function(libname, pkgname) {
  packageStartupMessage("This version changes the hclustcompro function behavior (updated from 1.2.4).\nYou can find the old version under the name perioclust or adapt your script to the new format of hclustcompro!")
}
