## Gestion des données manquantes

# TODO: function interface missMDA, Amelia, VIM
# missMDA: imputation simple ou multiple avec PCA avec NA
# Amelia: imputation multiple avec bootstrap
# VIM: Visualisation NA et combinaison de NA

