\name{optim_select_alpha}
\alias{optim_select_alpha}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Optimized selection of the weights for the sources (hclustcompro)
}
\description{
Description of the function.
}
\usage{
optim_select_alpha(data = list(), 
                   acc = 2, 
                   method = "complete", 
                   algo = "NLOPT_GN_ORIG_DIRECT_L"
)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{
liste des matrices de distances
}
  \item{acc}{
nombre de chiffre après la virgule
}
  \item{method}{
stratégie d'aggrégation utilisée par hclustcompro
}
  \item{algo}{
Algorithme utilisé par nloptr pour trouver les poids optimaux
}
}
\details{
See NLOPT website
}
\value{

\item{solution }{vector of weigths}
\item{res_nlopt }{object return by nloptr}

}

\author{
A. COULON

L. BELLANGER

P. HUSI
}

\examples{
## The function is currently defined as
}

