\name{cerardat_app}
\alias{cerardat_app}
\title{
  Launch the shiny application.
}
\description{
  see \link[SPARTAAS]{cerardat}.
  }
\usage{
  cerardat_app()
}

\value{No return value}

\author{
  A. COULON

  L. BELLANGER

  P. HUSI
}



\examples{
  \donttest{
    if(interactive()){
      cerardat_app()
    }
  }

}

