\name{hclust}
\alias{hclust}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Hierarchical clustering for multiple datasets (Compromised clustering).
}
\description{
Implementation of \link[stats:hclust]{stats::hclust} for dealing with multiple dissimilarities matrices.
We use \link[fastcluster:hclust]{fastcluster::hclust}, another implementation of hclust with faster algorithms.
}
\usage{
hclust(d, method = "complete", members = NULL, weight = NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{d}{
a dissimilarity structure as produced by dist or a list of dissimilarity structure.
}
  \item{method}{
the agglomeration method to be used. This should be (an unambiguous abbreviation of) one of "ward.D", "ward.D2", "single", "complete", "average" (= UPGMA), "mcquitty" (= WPGMA), "median" (= WPGMC) or "centroid" (= UPGMC).
}
  \item{members}{
NULL or a vector with length size of d. See \link[stats:hclust]{stats::hclust} ‘Details’ section.
}
  \item{weight}{
The parameter in order to generate a mixed matrix on which the classical hclust method is applied (same length as d). \cr Formula: \deqn{D_\alpha = w_1 D_1 + w_2 D_2 + ...}\cr where \eqn{w_1} is the first value of vector weight, \eqn{D_1} the first matrix of d and \eqn{D_\alpha} the mixed matrix used in hclust. Sum of weight must be 1.
}
}
\details{
Data fusion (parameter weigth): it is necessary to define the appropriate proportion for each data source. This is the first sensitive point of the method that the user has to consider. A tool is provided to help him in his decision see \link[SPARTAAS]{hclustcompro_select_alpha} (case with only 2 matrices) or \link[SPARTAAS]{optim_select_alpha}.
}
\value{
See \link[stats:hclust]{stats::hclust} or \link[fastcluster:hclust]{fastcluster::hclust}
}

\author{
The stats::hclust function is based on Fortran code contributed to STATLIB by F. Murtagh.
The fastcluster::hclust function implements hierarchical clustering with the same interface as hclust from the stats package but with much faster algorithms.

A. COULON

L. BELLANGER

P. HUSI
}

\examples{
data(datangkor)

#network stratigraphic data (Network)
network <- datangkor$stratigraphy

#contingency table
cont <- datangkor$contingency

#obtain the dissimilarities matrices
distance <- CAdist(cont, nPC = 11)
constraint <- adjacency(network)

D1 = distance
D2 = constraint

tree = hclust(d = list(D1, D2), method = "ward.D2", weight = c(0.7,0.3))
plot(tree, hang=-1)

}

